package com.qmth.cet.plug.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.cet.plug.contant.SystemConstant;
import com.qmth.cet.plug.service.LogicService;
import com.qmth.cet.plug.util.Result;
import com.qmth.cet.plug.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Objects;
import java.util.StringJoiner;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * cet外挂接口 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2025-06-13
 */
@Api(tags = "cet外挂接口Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_CET_PLUG)
@Validated
public class CetPlugController {
    private final static Logger log = LoggerFactory.getLogger(CetPlugController.class);

    @Value("${com.qmth.cet4.dir}")
    String cet4Dir;

    @Value("${com.qmth.cet6.dir}")
    String cet6Dir;

    @Value("${com.qmth.client.dir}")
    String clientDir;

    @Resource
    LogicService logicService;

    @ApiOperation(value = "测试")
    @RequestMapping(value = "/test", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "返回数据", response = Objects.class)})
    @Aac(auth = false)
    public Result test() throws IOException, InterruptedException {
        log.info("test is come in");
        long start = System.currentTimeMillis();
        StringJoiner stringJoiner = new StringJoiner("");
        String projectPath = System.getProperty("user.dir");
        int cpuNum = Runtime.getRuntime().availableProcessors();
        ExecutorService executor = Executors.newFixedThreadPool(Math.abs(cpuNum / 2));
        if (Objects.nonNull(cet4Dir)) {
            logicService.execFile(cet4Dir, projectPath, "cet4", executor);
        }
        if (Objects.nonNull(cet6Dir)) {
            logicService.execFile(cet6Dir, projectPath, "cet6", executor);
        }
        executor.shutdown();
        while (!executor.awaitTermination(1, TimeUnit.MINUTES)) {
            log.info("线程池没有关闭");
        }
        log.info("线程池已经关闭");
        long end = System.currentTimeMillis();
        stringJoiner.add("============耗时============:").add((end - start) / 1000 + "").add("秒");
        log.info("request:{}", stringJoiner.toString());
        return ResultUtil.ok();
    }
}
